'use strict';

exports.__esModule = true;
exports.getContentFromUrl = exports.getAst = undefined;

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * get file ast
 */

var getAst = exports.getAst = function () {
  var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee(instance, content, fn, filepath) {
    var astCacheInstance, cacheKey, cacheData, debug, ast;
    return _regenerator2.default.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            astCacheInstance = null, cacheKey = '';
            // get ast from cache

            if (!(instance.stc.config.cache !== false)) {
              _context.next = 12;
              break;
            }

            astCacheInstance = getAstCacheInstance(instance.stc, instance.file.extname);
            cacheKey = (0, _stcHelper.md5)(instance.stc.config._tplCacheKey + content + filepath);
            _context.next = 6;
            return astCacheInstance.get(cacheKey);

          case 6:
            cacheData = _context.sent;

            if (!cacheData) {
              _context.next = 12;
              break;
            }

            debug = instance.stc.debug('cache');

            debug('getAst from cache, file is `' + instance.file.path + '`');
            instance.file.setAst(cacheData);
            return _context.abrupt('return', cacheData);

          case 12:
            _context.next = 14;
            return fn();

          case 14:
            ast = _context.sent;

            if (!astCacheInstance) {
              _context.next = 18;
              break;
            }

            _context.next = 18;
            return astCacheInstance.set(cacheKey, ast);

          case 18:
            return _context.abrupt('return', ast);

          case 19:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getAst(_x, _x2, _x3, _x4) {
    return _ref.apply(this, arguments);
  };
}();
/**
 * get cache instance
 */


/**
 * get content from url
 */

var getContentFromUrl = exports.getContentFromUrl = function () {
  var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2(url) {
    var fn, result;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            fn = (0, _stcHelper.promisify)(_request2.default.get, _request2.default);
            _context2.next = 3;
            return fn(url, {
              method: 'get',
              strictSSL: false,
              timeout: 20 * 1000,
              encoding: null,
              headers: {
                'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
                'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_11_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.113 Safari/537.36',
                'Accept-Language': 'zh-CN,zh;q=0.8,en;q=0.6,ja;q=0.4,nl;q=0.2,zh-TW;q=0.2'
              }
            });

          case 3:
            result = _context2.sent;
            return _context2.abrupt('return', result.body);

          case 5:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function getContentFromUrl(_x6) {
    return _ref2.apply(this, arguments);
  };
}();

exports.getAstCacheInstance = getAstCacheInstance;
exports.checkRunIsExecute = checkRunIsExecute;
exports.checkInMaster = checkInMaster;
exports.getCacheInstance = getCacheInstance;
exports.getConcurrentLimitInstance = getConcurrentLimitInstance;
exports.getAwaitInstance = getAwaitInstance;

var _cluster = require('cluster');

var _stcHelper = require('stc-helper');

var _concurrent_limit = require('./concurrent_limit.js');

var _concurrent_limit2 = _interopRequireDefault(_concurrent_limit);

var _stcAwait = require('stc-await');

var _stcAwait2 = _interopRequireDefault(_stcAwait);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * concurrent limit task instances
 */
var concurrentLimitInstances = {};
/**
 * await instances
 */
var awaitInstances = {};

/**
 * get ast cache instance
 */
function getAstCacheInstance(stc, extname) {
  var astCacheKey = '__ast__';
  var astCacheInstance = stc.cacheInstances[astCacheKey];
  if (!astCacheInstance) {
    astCacheInstance = new stc.cache({
      path: stc.config.cachePath,
      type: (stc.config.product || 'default') + '/ast/' + extname
    });
    stc.cacheInstances[astCacheKey] = astCacheInstance;
  }
  return astCacheInstance;
}

/**
 * check run method is execute
 */
function checkRunIsExecute(instance, method) {
  if (!instance.prop('__isRun__')) {
    throw new Error(method + ' only allow invoked in update method');
  }
}

/**
 * check in master
 */
function checkInMaster(method) {
  if (!_cluster.isMaster) {
    throw new Error(method + ' method must be invoked in master');
  }
}function getCacheInstance(plugin) {
  var onlyMemory = arguments.length <= 1 || arguments[1] === undefined ? true : arguments[1];

  var md5Value = plugin.getMd5() + onlyMemory ? 'true' : 'false';
  if (!plugin.stc.cacheInstances[md5Value]) {
    plugin.stc.cacheInstances[md5Value] = new plugin.stc.cache({
      path: plugin.config.cachePath,
      onlyMemory: onlyMemory
    });
  }
  return plugin.stc.cacheInstances[md5Value];
}

/**
 * get concurrent limit instance
 */
function getConcurrentLimitInstance(limit, ignoreErrorFn, key) {
  if (!(key in concurrentLimitInstances)) {
    concurrentLimitInstances[key] = new _concurrent_limit2.default(limit, ignoreErrorFn);
  }
  return concurrentLimitInstances[key];
}
/**
 * get await instance
 */
function getAwaitInstance(key) {
  if (!(key in awaitInstances)) {
    awaitInstances[key] = new _stcAwait2.default();
  }
  return awaitInstances[key];
}
//# sourceMappingURL=helper.js.map